package com.ruoyi.warehouseBusiness.domain;

import com.ruoyi.common.core.domain.BaseEntity;

import java.math.BigDecimal;

/**
 * 仓储明细电子标签从表
 * @author ni
 * @date 2022-04-26
 */
public class TWarehousebillsitemsElabel extends BaseEntity {

    /** 表 id */
    private Long fId;
    /** fGPID，对应明细从表的主表id */
    private Long fGPid;
    /** fPID对应明细从表id */
    private Long fPid;
    /** 电子标签内容 */
    private String fContent;
    /** 件数 */
    private Long fQty;
    /** 重量 */
    private BigDecimal fWeight;
    /** 1入库 2出库 3库存总账 */
    private Integer fType;
    /** 0在库 1出库 */
    private Integer fStatus;

    public Long getfId() {
        return fId;
    }

    public void setfId(Long fId) {
        this.fId = fId;
    }

    public Long getfGPid() {
        return fGPid;
    }

    public void setfGPid(Long fGPid) {
        this.fGPid = fGPid;
    }

    public Long getfPid() {
        return fPid;
    }

    public void setfPid(Long fPid) {
        this.fPid = fPid;
    }

    public String getfContent() {
        return fContent;
    }

    public void setfContent(String fContent) {
        this.fContent = fContent;
    }

    public Long getfQty() {
        return fQty;
    }

    public void setfQty(Long fQty) {
        this.fQty = fQty;
    }

    public BigDecimal getfWeight() {
        return fWeight;
    }

    public void setfWeight(BigDecimal fWeight) {
        this.fWeight = fWeight;
    }

    public Integer getfType() {
        return fType;
    }

    public void setfType(Integer fType) {
        this.fType = fType;
    }

    public Integer getfStatus() {
        return fStatus;
    }

    public void setfStatus(Integer fStatus) {
        this.fStatus = fStatus;
    }
}
